/*********************************************************************************************
 * (c) COPYRIGHT 2021 by NetModule AG, Switzerland
 * -------------------------------------------------------------------------------------------
 *
 * InfoWriter MODULE
 *
 * -------------------------------------------------------------------------------------------
 *
 * FILENAME      : InfoWriter.c
 * CREATION DATE : Nov 18, 2021
 * TARGET        : embedded Linux
 *
 *********************************************************************************************/

#include <stdlib.h>                      // for calloc, free

#include <nmapp/exception/exception.h>
#include <nmapp/sys/assertion.h>

#include "InfoWriter-private.h"


/*********************************************************************************************
 *        LOCALS
 *********************************************************************************************/

//--------------------------------------------------------------------------------------------
void setupDirectoryTree(InfoWriterRef me)
{
    char        partialPath[INFO_DATA_PATH_SIZE] = "";
    uint32_t    dirPermissions = (S_IRWXU|S_IRGRP|S_IXGRP|S_IROTH|S_IXOTH);
    size_t      pos = 1;
    const char* tmpPathStr = &me->fileName[pos];
    char*       startPart  = strchr(tmpPathStr, '/');

    ILog_msg(me->logRef, LOG_LEVEL_DBG, "setting up directories for '%s'...", me->fileName);
    while(startPart != NULL)
    {
        // prepare tmpPathStr for next directory-tree level:
        pos = (size_t)((startPart - me->fileName) + 1);
        tmpPathStr = &me->fileName[pos];
        startPart = strchr(tmpPathStr, '/');

        // build path until pos and create dir unless existing:
        snprintf(&partialPath[0], pos, "%s", me->fileName);
        partialPath[pos] = '\0';
        bool isPathExisting = IFileOperation_isExist(me->fileOpItf, (const char*)partialPath);
        if(!isPathExisting || (isPathExisting && !IFileOperation_isDirectory(me->fileOpItf, (const char*)partialPath)))
        {
            ILog_msg(me->logRef, LOG_LEVEL_DBG, "path '%s' does not exist or/and is not a directory --> creating", partialPath);
            IFileOperation_createDirectory(me->fileOpItf, (const char*)partialPath, dirPermissions);
        }
    }
}
//--------------------------------------------------------------------------------------------
void setupDynamicFileContent(char *pDataBuf, uint32_t maxBufSize, DynamicInfo_t* pDynamicInfo)
{
    if(pDynamicInfo->lifeTime.percentUsedSlc != -1)
    {
        snprintf(pDataBuf, maxBufSize, INFO_STRING_DYNAMIC_DATA_FMT,
                                             (int)pDynamicInfo->preEolValue, pDynamicInfo->preEolString,
                                             (int)pDynamicInfo->lifeTime.percentUsedSlc,
                                             (int)pDynamicInfo->lifeTime.percentUsedMlc,
                                             (float)pDynamicInfo->freeDiskSpaceMB);
        return;
    }

    // eMMC does not support lifetime values
    snprintf(pDataBuf, maxBufSize, INFO_STRING_DYNAMIC_DATA_UNSUPPORTED_LIFETIMES_FMT,
                                     (int)pDynamicInfo->preEolValue, pDynamicInfo->preEolString,
                                     (float)pDynamicInfo->freeDiskSpaceMB);
}

/*********************************************************************************************
 *        GLOBALS
 *********************************************************************************************/
//--------------------------------------------------------------------------------------------
InfoWriterRef InfoWriter_create(ILogRef logRef, IFileOperationRef fileOpRef, const char* file)
{
    assert(fileOpRef != NULL);
    assert(file != NULL);

    InfoWriterRef me = (InfoWriterRef)calloc(1, sizeof(InfoWriterPrivate_t));
    if(me == NULL)
    {
        Throw(EX_CREATE, "Creating InfoCollector failed");
    }
    me->logRef    = logRef;
    me->fileOpItf = fileOpRef;
    me->fileName  = file;

    return me;
}
//--------------------------------------------------------------------------------------------
void InfoWriter_destroy(InfoWriterRef me)
{
    if(me != NULL)
    {
        free(me);
    }
}
//--------------------------------------------------------------------------------------------
void InfoWriter_writeStaticInfoToFile(InfoWriterRef me, StaticInfo_t* pStaticInfo)
{
    assert(me != NULL);
    assert(pStaticInfo != NULL);
    char dataBuffer[INFO_DATA_SIZE] = "";
    snprintf(&dataBuffer[0], INFO_DATA_SIZE, INFO_STRING_STATIC_DATA_FMT,
                                                 pStaticInfo->mmcChipName,
                                                 pStaticInfo->mmcHwRevision,
                                                 pStaticInfo->mmcRevision,
                                                 pStaticInfo->mmcFwVersion,
                                                 pStaticInfo->mmcSerialNbr,
                                                 (int)(pStaticInfo->isReliablyWritable ? 1 : 0),
                                                 pStaticInfo->reliableWriteConfig.flags);
    size_t dataSize = strlen(dataBuffer);
    setupDirectoryTree(me);
    IFileOperation_open(me->fileOpItf, me->fileName, "w+");
    IFileOperation_writeToFile(me->fileOpItf, &dataBuffer[0], (uint32_t)sizeof(dataBuffer));
    IFileOperation_close(me->fileOpItf);
    me->staticDataFileSize = (uint32_t)dataSize;
    ILog_msg(me->logRef, LOG_LEVEL_DBG, "static information written to '%s'", me->fileName);
}
//--------------------------------------------------------------------------------------------
void InfoWriter_writeDynamicInfoToFile(InfoWriterRef me, DynamicInfo_t* pDynamicInfo)
{
    assert(me != NULL);
    assert(pDynamicInfo != NULL);
    if(!IFileOperation_isExist(me->fileOpItf, me->fileName))
    {
        Throw(EX_MISSING, "Something went wrong: File %s is missing", me->fileName);
    }

    char dataBuffer[INFO_DATA_SIZE] = "";
    setupDynamicFileContent(&dataBuffer[0], (uint32_t)INFO_DATA_SIZE, pDynamicInfo);
    IFileOperation_open(me->fileOpItf, me->fileName, "r+");
    IFileOperation_writeUpdateToFile(me->fileOpItf, true, me->staticDataFileSize, &dataBuffer[0], (uint32_t)sizeof(dataBuffer));
    IFileOperation_close(me->fileOpItf);
    ILog_msg(me->logRef, LOG_LEVEL_DBG, "dynamic information written to '%s'", me->fileName);
}

