/*********************************************************************************************
 * (c) COPYRIGHT 2021 by NetModule AG, Switzerland
 * -------------------------------------------------------------------------------------------
 *
 * storage-detector MODULE
 *
 * -------------------------------------------------------------------------------------------
 *
 * FILENAME      : storage-detector.c
 * CREATION DATE : Oct 28, 2021
 * TARGET        : embedded Linux
 *
 *********************************************************************************************/

#include <unistd.h>               // for readlink
#include <errno.h>                // for errno
#include <string.h>               // for strerror, ...
#include <stdio.h>                // for sscanf

#include <nmapp/exception/exception.h>
#include <nmapp/sys/assertion.h>

#include "StorageDetector-private.h"

/*********************************************************************************************
 *        LOCALS
 *********************************************************************************************/

// for singleton:
StorageDetector_t  StorageDetector;
static StorageDetectorRef localMe = NULL;


//--------------------------------------------------------------------------------------------
void getLinkTargetOfStorage(StorageDetectorRef me, const char *linkPath, char* linkTarget, uint32_t maxLinkTargetLen)
{
    ssize_t linkTargetLen = readlink(linkPath, linkTarget, maxLinkTargetLen);
    if(linkTargetLen == -1)
    {
        Throw(EX_READ, "Read target link of %s failed: %s", linkPath, strerror(errno));
    }
    linkTarget[linkTargetLen] = '\0';
    if(linkTargetLen == maxLinkTargetLen)
    {
        Throw(EX_LIMIT_CHECK, "Target link (%s) might be too long (maxLen=%d)", linkTarget, (int)maxLinkTargetLen);
    }
}
//--------------------------------------------------------------------------------------------
void buildDeviceInformationPath(StorageDetectorRef me)
{
    int blockNbr = 0;
    int match = sscanf((const char*)me->mmcDevice, "/dev/mmcblk%d", &blockNbr);
    if((match == EOF) || (match != 1))
    {
        Throw(EX_READ, "Read block number from %s", me->mmcDevice);
    }
    snprintf(&me->mmcDeviceInfoPath[0], (MMCDEV_BUF_SIZE/2), "/sys/class/mmc_host/mmc%d/mmc%d", blockNbr, blockNbr);
    strncat(&me->mmcDeviceInfoPath[0], ":0001", MMCDEV_BUF_SIZE);
}


/*********************************************************************************************
 *        GLOBALS
 *********************************************************************************************/
//--------------------------------------------------------------------------------------------
StorageDetectorRef StorageDetector_create(ILogRef logRef)
{
    if(localMe != NULL)
    {
        ILog_msg(localMe->logRef, LOG_LEVEL_WARN, "Singleton of StorageDetector is created multiple times");
        return localMe;
    }
    memset(&StorageDetector, 0, sizeof(StorageDetector_t));
    localMe = (StorageDetectorRef)&StorageDetector;
    localMe->logRef = logRef;
    localMe->getLinkTargetPathFn = &getLinkTargetOfStorage;

    ILog_msg(localMe->logRef, LOG_LEVEL_DBG, "Created StorageDetector");
    return localMe;
}
//--------------------------------------------------------------------------------------------
void StorageDetector_destroy(StorageDetectorRef me)
{
    if(localMe != NULL)
    {
        assert(me == localMe);
        memset(localMe, 0, sizeof(StorageDetector_t));
        localMe = NULL;
    }
}
//--------------------------------------------------------------------------------------------
void StorageDetector_init(StorageDetectorRef me)
{
    assert(me != NULL);
    const size_t maxLinkBufSize = (MMCDEV_BUF_SIZE/2);
    const char* storageZeroDevice = "/dev/storage0";
    uint32_t maxDevPathLen = (uint32_t)(maxLinkBufSize-1);  // keep 1 byte for the terminating NULL byte
    char linkTarget[maxLinkBufSize];
    me->getLinkTargetPathFn(me, storageZeroDevice, &linkTarget[0], maxDevPathLen);
    snprintf(&me->mmcDevice[0], MMCDEV_BUF_SIZE, "/dev/%s", &linkTarget[0]);
    buildDeviceInformationPath(me);
}
//--------------------------------------------------------------------------------------------
const char* StorageDetector_getMmcDevice(StorageDetectorRef me)
{
    assert(me != NULL);
    return (const char*)&me->mmcDevice[0];
}
//--------------------------------------------------------------------------------------------
const char* StorageDetector_getMmcDeviceInfoPath(StorageDetectorRef me)
{
    assert(me != NULL);
    return (const char*)&me->mmcDeviceInfoPath[0];
}


